@@ -52,29 +52,30 @@ updated.model <- function(model, new.y = NULL, new.data = NULL){
   # Extract formulas and data
   mod.fixd <- as.formula(model$call$fixed)
   mod.rand <- model$call$random
+  mod.data <- model$data
+  if(!is.null(model$na.action) && model$na.action == 8) mod.data <- na.omit(mod.data)
   
   if(is.null(new.data)){
     # Place ystars in data
-    mod.data <- model$data
     mod.data[,as.character(mod.fixd[[2]])] <- unname(new.y)
-  } else{
-    mod.data <- new.data
-  }
+  } 
   
   # create new lme
   ctrl <- nlme::lmeControl(opt = 'optim', returnObject = TRUE)
-  error <- NULL
+  
   if(is.null(mod.rand)){
-    out.lme <- catchr::catch_expr(
-      do.call("lme", args = list(fixed = mod.fixd, data = mod.data, control = ctrl)), 
-      warning, message, error
+    f1 <- factory(
+      function(mod.fixd, mod.data, ctrl) 
+        do.call("lme", args = list(fixed = mod.fixd, data = mod.data, control = ctrl))
     )
+    out.lme <- f1(mod.fixd, mod.data, ctrl)
   } else{
     mod.rand <- as.formula(mod.rand)
-    out.lme <- catchr::catch_expr(
-      do.call("lme", args = list(fixed = mod.fixd, data = mod.data, random = mod.rand, control = ctrl)), 
-      warning, message, error
+    f1 <- factory(
+      function(mod.fixd, mod.data, mod.rand, ctrl) 
+        do.call("lme", args = list(fixed = mod.fixd, data = mod.data, random = mod.rand, control = ctrl))
     )
+    out.lme <- f1(mod.fixd, mod.data, mod.rand, ctrl)
   }
   
   out.lme
@@ -91,12 +92,9 @@ refit_lme <- function(ystar = NULL, model, .f) {
   if(!is.null(ystar))
     refits <- purrr::map(ystar, function(y) updated.model(model = model, new.y = y))
   
-  stats <- purrr::map(refits, ~.f(.x$value))
-  warn  <- lapply(refits, function(.x) unlist(.x$warning)$message)
-  msgs  <- lapply(refits, function(.x) unlist(.x$message)$message)
-  errs  <- lapply(refits, function(.x) unlist(.x$error)$message)
+  stats <- purrr::map(refits, ~.f(.x))
   
-  list(tstar = stats, warnings = list(warning = warn, message = msgs, error = errs))
+  list(tstar = stats, warnings = collect_warnings(stats))
 }
 
 
